from django.urls import path, reverse
from wagtail import hooks
from wagtail.admin import widgets as wagtail_admin_widgets

from .views import mailchimp_integration_view


@hooks.register('register_admin_urls')
def urlconf_wagtail_mailchimp():
    return [
        path('mailchimp-integration/<int:page_id>', mailchimp_integration_view, name="mailchimp_integration_view"),
    ]


@hooks.register('register_page_listing_buttons')
def page_listing_buttons(page, page_perms, next_url=None):
    if hasattr(page, "is_mailchimp_integration") and hasattr(page, "audience_list_id"):
        if page.audience_list_id:
            url = reverse("mailchimp_integration_view", args=[page.pk, ])
            yield wagtail_admin_widgets.PageListingButton(
                "Mailchimp Integration",
                url,
                priority=50
            )



import json
from datetime import date

from django.contrib import messages
from django.core.mail import mail_admins
from django.forms.forms import NON_FIELD_ERRORS
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.views.generic import FormView
from mailchimp3.mailchimpclient import MailChimpError
from modelcluster.models import get_all_child_relations
from wagtail.contrib.forms.models import AbstractFormField
from wagtail.models import Page

from .api import MailchimpApi
from .forms import MailChimpForm, MailchimpIntegrationForm
from .models import MailchimpSettings


class MailChimpView(FormView):
    """
    Displays and processes a form based on a MailChimp list.
    """
    form_class = MailChimpForm
    page_instance = None
    merge_fields = None
    interest_categories = None
    api = None

    def get_api(self):
        if self.api:
            return self.api
        mc_settings = MailchimpSettings.for_request(self.request)
        self.api = MailchimpApi(api_key=mc_settings.api_key)
        return self.api

    def get_clean_merge_fields(self, form):
        """
        Returns dictionary of MailChimp merge variables with cleaned
        form values.

        :param form: the form instance.
        :rtype: dict.
        """
        merge_fields = {}

        # Add merge variable values.
        for merge_field in self.get_merge_fields():
            mc_type = merge_field.get('type', '')
            name = merge_field.get('tag', '')
            value = form.cleaned_data.get(name, '')

            # Assemble address components into a single string value per
            # http://kb.mailchimp.com/lists/growth/format-list-fields#Address.
            if mc_type == 'address':
                values = []
                for f in ['addr1', 'addr2', 'city', 'state', 'zip', 'country']:
                    key = '{0}-{1}'.format(name, f)
                    val = form.cleaned_data.get(key)
                    if val:
                        values.append(val)
                value = '  '.join(values)

            # Convert date to string.
            if mc_type == 'date' and isinstance(value, date):
                value = value.strftime('%m/%d/%Y')

            # Convert birthday to string.
            if mc_type == 'birthday' and isinstance(value, date):
                value = value.strftime('%m/%d')

            if value:
                merge_fields.update({name: value})

        return merge_fields

    def get_context_data(self, **kwargs):
        """
        Returns view context data dictionary.

        :rtype: dict.
        """
        context = super(MailChimpView, self).get_context_data(**kwargs)
        page = self.page_instance
        context.update({'self': page, 'page': page, })

        return context

    def get_interest_categories(self):
        """
        Returns list of MailChimp grouping dictionaries.

        :rtype: dict.
        """

        api = self.get_api()

        if self.interest_categories is None and api.is_active:
            list_id = self.page_instance.list_id

            interest_categories = api.get_interest_categories_for_list(list_id=list_id)

            categories = []

            for category in interest_categories:
                category_id = category.get('id', '')

                interest_category = {
                    "id": category_id,
                    "title": category.get('title', ''),
                    'type': category.get('type', '')
                }

                interests = api.get_interests_for_interest_category(list_id=list_id,
                                                                    interest_category_id=category_id)

                interest_category['interests'] = interests

                categories.append(interest_category)

            self.interest_categories = categories

        return self.interest_categories

    def get_merge_fields(self):

        """
        Returns list of MailChimp merge fields dictionaries.

        :rtype: dict.
        """

        api = self.get_api()

        if self.merge_fields is None and api.is_active:
            self.merge_fields = api.get_merge_fields_for_list(self.page_instance.list_id)

        # If we don't have any merge variables to build a form from,
        # raise an HTTP 404 error.
        if not self.merge_fields:
            raise Http404

        return self.merge_fields

    def get_form(self, form_class=None):
        """
        Returns MailChimpForm instance.

        :param form_class: name of the form class to use.
        :rtype: MailChimpForm.
        """

        merge_fields = self.get_merge_fields()
        interest_categories = self.get_interest_categories()

        return MailChimpForm(merge_fields, interest_categories, **self.get_form_kwargs())

    def get_template_names(self):
        """
        Returns list of available template names.

        :rtype: list.
        """
        return [self.page_instance.get_template(self.request)]

    def form_valid(self, form):

        """
        Subscribes to MailChimp list if form is valid.

        :param form: the form instance.
        """

        api = self.get_api()

        # Subscribe to the MailChimp list.
        clean_merge_fields = self.get_clean_merge_fields(form)

        # raise Exception(clean_merge_vars)

        status = "subscribed"

        if self.page_instance.double_optin:
            status = 'pending'

        clean_interests = form.cleaned_data.get('INTERESTS', [])

        interests_payload = {}

        for interest in clean_interests:
            interests_payload[interest] = True

        error_traceback = None

        context = {'page': self.page_instance, 'self': self.page_instance}

        # Must have an email address.
        if api.is_active and 'EMAIL' in clean_merge_fields:
            data = {
                'email_address': clean_merge_fields.pop('EMAIL'),
                'merge_fields': clean_merge_fields,
                'status': status,
            }

            if interests_payload:
                data['interests'] = interests_payload

            if self.page_instance.success_redirect_page:
                self.success_url = self.page_instance.success_redirect_page.url

            try:
                list_id = self.page_instance.list_id
                api.add_user_to_list(list_id=list_id, data=data)
            except MailChimpError as e:
                error_traceback = e
                if e.args and e.args[0]:
                    error = e.args[0]
                    if error['title']:
                        if error['title'] == "Member Exists":
                            if self.success_url:
                                messages.add_message(self.request, messages.INFO,
                                                     "You are already subscribed to our mailing list. Thank you!")
                                return super(MailChimpView, self).form_valid(form)
                            else:
                                context.update({
                                    "success_message": "You are already subscribed to our mailing list. Thank you!",
                                    "form": self.get_form(),
                                })
                                return self.render_to_response(context)

            except Exception as e:
                error_traceback = e
        else:
            if not api.is_active:
                error_traceback = "MAILCHIMP API not active"
            else:
                error_traceback = "No email in fields"

        if error_traceback:
            mail_admins("Error adding user to mailing list", str(error_traceback), fail_silently=True)

            form.errors[NON_FIELD_ERRORS] = form.error_class(
                ["We are having issues adding you to our mailing list. Please try later"]
            )
            return super(MailChimpView, self).form_invalid(form)

        default_success_message = "You have been successfully added to our mailing list"

        if self.success_url:
            if self.page_instance.thank_you_text:
                message = self.page_instance.thank_you_text
            else:
                message = default_success_message

            messages.add_message(self.request, messages.INFO, message)

            return super(MailChimpView, self).form_valid(form)

        context.update({
            "success_message": "You have been successfully added to our mailing list",
            "form": self.get_form(),
        })

        return self.render_to_response(context)


def mailchimp_integration_view(request, page_id):
    page = Page.objects.get(pk=page_id)
    form_page = page.specific
    edit_url = reverse("wagtailadmin_pages:edit", args=[form_page.pk])
    context = {"page": form_page, "page_edit_url": edit_url}
    template_name = "wagtailmailchimp/mailchimp_integration_form.html"

    parent_page = form_page.get_parent()
    explore_url = reverse("wagtailadmin_explore", args=[parent_page.id])

    form_fields_rel_name = None
    # get form fields relation name
    relations = get_all_child_relations(form_page)
    for relation in relations:
        related_name = relation.related_name
        rels = getattr(form_page, related_name)
        # check if is instance of AbstractFormField
        if isinstance(rels.first(), AbstractFormField):
            form_fields_rel_name = related_name
            break

    merge_fields = None
    form_fields = None
    has_form_fields = False
    interest_categories = None

    if form_fields_rel_name and hasattr(form_page, form_fields_rel_name):
        form_fields = getattr(form_page, form_fields_rel_name).all()
        mc_settings = MailchimpSettings.for_request(request)
        api = MailchimpApi(api_key=mc_settings.api_key)
        lists = api.get_lists()

        for audience in lists:
            if audience.get("id") == form_page.audience_list_id:
                context.update({"audience": audience})
                break
        merge_fields = api.get_merge_fields_for_list(form_page.audience_list_id)
        interest_categories = api.get_interests_for_list(form_page.audience_list_id)

    if form_fields is not None:
        has_form_fields = True

    context.update({"has_form_fields": has_form_fields})

    if request.method == 'POST':
        form = MailchimpIntegrationForm(merge_fields=merge_fields, form_fields=form_fields, data=request.POST)

        if form.is_valid():
            merge_fields_data = json.dumps(form.cleaned_data)
            interest_categories_data = json.dumps(interest_categories)
            form_page.merge_fields_mapping = merge_fields_data
            form_page.interest_categories = interest_categories_data
            form_page.save()

            return HttpResponseRedirect(explore_url)
        else:
            context.update({"form": form})
            return render(request, template_name, context=context)

    initial_data = None
    if form_page.merge_fields_mapping:
        try:
            initial_data = json.loads(form_page.merge_fields_mapping)
        except Exception:
            pass

    form = MailchimpIntegrationForm(merge_fields=merge_fields, form_fields=form_fields, initial=initial_data)
    context.update({"form": form})

    return render(request, template_name, context=context)



import json

from django.contrib import messages
from django.db import models
from django.forms import BooleanField
from django.template import Context, Template
from django.template.response import TemplateResponse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from mailchimp3.mailchimpclient import MailChimpError
from wagtail.admin.panels import FieldPanel, FieldRowPanel, MultiFieldPanel
from wagtail.contrib.forms.models import AbstractForm
from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.contrib.settings.registry import register_setting

from .api import MailchimpApi
from .widgets import MailchimpSubscriberOptinWidget, MailchimpAudienceListWidget


@register_setting
class MailchimpSettings(BaseSiteSetting):
    api_key = models.CharField(verbose_name=_("Mailchimp API Key"), max_length=50, blank=True, null=True,
                               help_text=_("Mailchimp API Key "))
    default_audience_id = models.CharField(verbose_name=_("Default Mailchimp Audience Id"), max_length=100, blank=True,
                                           null=True,
                                           help_text=_("Default Mailchimp Audience Id"))

    panels = [
        FieldPanel("api_key"),
        FieldPanel("default_audience_id"),
    ]


class AbstractMailChimpPage(models.Model):
    """
    Abstract MailChimp page definition.
    """
    list_id = models.CharField(_('MailChimp Audience'), max_length=50,
                               help_text=_('Select MailChimp Audience to use for this form'))
    double_optin = models.BooleanField(_('Double Opt-In'), default=False,
                                       help_text=_(
                                           'Check to use double opt-in process for new subscribers. '
                                           'If enabled, users must confirm their subscription via '
                                           'an email sent by MailChimp'))

    success_redirect_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Success Redirect Page"),
        help_text=_(
            "Page to redirect to after successful submission.Leave unselected to show this page with an empty form")
    )

    thank_you_text = models.TextField(blank=True, null=True, help_text=_("Message to show on successful submission"),
                                      verbose_name=_("Thank you text"))

    class Meta(object):
        abstract = True

    def serve(self, request):
        """
        Serves the page as a MailChimpView.

        :param request: the request object.
        :rtype: django.http.HttpResponse.
        """

        from .views import MailChimpView

        view = MailChimpView.as_view(page_instance=self)
        return view(request)

    content_panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('list_id', widget=MailchimpAudienceListWidget),
                FieldPanel('double_optin'),
            ], classname='label-above'),
            FieldPanel('success_redirect_page')
        ], (_('MailChimp Settings'))),
        FieldPanel('thank_you_text')
    ]


class AbstractMailchimpIntegrationForm(AbstractForm):
    class Meta:
        abstract = True

    mailchimp_field_name = 'mailchimp_subscribe_check'
    mailchimp_interests_field_name = 'mailchimp_interests_check'
    default_mailing_list_checkbox_label = _("Join Our Mailing List")

    audience_list_id = models.CharField(max_length=50, blank=True, null=True, verbose_name=_('MailChimp Audience'),
                                        help_text=_('Select MailChimp Audience to add users to'))
    merge_fields_mapping = models.TextField(blank=True, null=True)
    interest_categories = models.TextField(blank=True, null=True)

    mailing_list_checkbox_label = models.CharField(max_length=200, blank=True,
                                                   verbose_name=_("Mailing list checkbox label"))

    integration_panels = [
        FieldPanel("audience_list_id", widget=MailchimpAudienceListWidget),
    ]

    is_mailchimp_integration = True

    def remove_mailchimp_field(self, form):
        form.fields.pop(self.mailchimp_field_name, None)
        return form.cleaned_data.pop(self.mailchimp_field_name, None)

    def process_form_submission(self, form, request=None):

        self.remove_mailchimp_field(form)

        form_submission = super(AbstractMailchimpIntegrationForm, self).process_form_submission(form)

        try:
            self.post_process_submission(form, form_submission)
        except Exception as e:
            pass

        return form_submission

    def post_process_submission(self, form, form_submission):
        pass

    def should_process_form(self, request, form_data):
        return True

    def render_landing_page(self, request, form_submission=None, *args, form_context=None, **kwargs):
        context = self.get_context(request)
        context['form_submission'] = form_submission
        if form_context:
            context.update(form_context)

        return TemplateResponse(
            request,
            self.get_landing_page_template(request),
            context
        )

    @method_decorator(csrf_exempt)
    def serve(self, request, *args, **kwargs):
        if request.method == 'POST':
            form = self.get_form(request.POST, request.FILES, page=self, user=request.user)

            if form.is_valid():
                form_submission = None
                hide_thank_you_text = False
                if self.should_process_form(request, form.cleaned_data):
                    form_submission = self.process_form_submission(form)
                    form_data = dict(form.data)
                    user_checked_sub = bool(form_data.get(self.mailchimp_field_name, False))
                    user_selected_interests = form_data.get(self.mailchimp_interests_field_name, None)

                    if user_checked_sub:
                        self.integration_operation(self, form=form, request=request,
                                                   user_selected_interests=user_selected_interests)
                else:
                    # we have an issue with the form submission.Don't show thank you text
                    hide_thank_you_text = True

                return self.render_landing_page(request, form_submission,
                                                *args,
                                                form_context={'hide_thank_you_text': hide_thank_you_text},
                                                **kwargs)

        form = self.get_form(page=self, user=request.user)
        context = self.get_context(request)

        if self.has_list_id_and_email:
            form.fields[self.mailchimp_field_name] = BooleanField(
                label='',
                required=False,
                widget=MailchimpSubscriberOptinWidget(
                    interest_categories=self.get_data().get("interest_categories", []),
                    label=self.mailing_list_checkbox_label or self.default_mailing_list_checkbox_label,
                    interests_field_name=self.mailchimp_interests_field_name
                )
            )

        context['form'] = form
        return TemplateResponse(
            request,
            self.get_template(request),
            context
        )

    def integration_operation(self, instance, **kwargs):
        request = kwargs.get('request', None)

        mc_settings = MailchimpSettings.for_request(request)

        mailchimp = MailchimpApi(api_key=mc_settings.api_key)

        user_selected_interests = kwargs.get('user_selected_interests', None)

        rendered_dictionary = self.render_dictionary(
            self.format_form_submission(kwargs['form']),
            user_selected_interests=user_selected_interests
        )

        try:
            dict_data = json.loads(rendered_dictionary)
            list_id = self.audience_list_id
            mailchimp.add_user_to_list(list_id=list_id, data=dict_data)
            if request:
                messages.add_message(request, messages.INFO,
                                     'You have been successfully added to our mailing list!')
        except MailChimpError as e:
            if request:
                if e.args and e.args[0]:
                    error = e.args[0]
                    if error['title']:
                        if error['title'] == "Member Exists":
                            messages.add_message(request, messages.INFO,
                                                 "You are already subscribed to our mailing list. Thank you!")
                    else:
                        messages.add_message(
                            request, messages.ERROR,
                            "You have successfully registered this event, but we are having issues"
                            " adding you to our mailing list. We will try to add you later")
        except Exception as e:
            if request:
                messages.add_message(
                    request, messages.ERROR,
                    "We are having issues adding you to our mailing list. We will try to add you later")

    def format_form_submission(self, form):
        formatted_form_data = {}

        for k, v in form.cleaned_data.items():
            formatted_form_data[k.replace('-', '_')] = v
        return formatted_form_data

    def get_data(self):
        data = {
            "email_field": None,
            "merge_fields": {},
            "interest_categories": {}
        }

        merge_fields_mapping = {}
        if self.merge_fields_mapping:
            try:
                merge_fields_mapping = json.loads(self.merge_fields_mapping)
            except Exception:
                pass

        interest_categories = {}
        if self.interest_categories:
            try:
                interest_categories = json.loads(self.interest_categories)
            except Exception:
                pass

        for key, value in merge_fields_mapping.items():
            if key == "EMAIL":
                data.update({"email_field": merge_fields_mapping.get("EMAIL")})
            else:
                data["merge_fields"].update({key: value})

        data.update({"interest_categories": interest_categories})

        return data

    def get_merge_fields(self):
        if 'merge_fields' in self.get_data():
            return self.get_data()['merge_fields']
        return {}

    def get_email_field_template(self):
        return "{}{}{}".format("{{", self.get_data()['email_field'], "}}")

    def get_merge_fields_template(self):
        fields = self.get_merge_fields()
        for key, value in fields.items():
            if value:
                fields[key] = "{}{}{}".format("{{", value, "}}")
        return fields

    @property
    def has_list_id_and_email(self):
        return self.audience_list_id and self.get_email_address()

    def get_email_address(self):
        if "email_field" in self.get_data():
            return self.get_data().get("email_field")
        return None

    def combine_interest_categories(self):
        interest_dict = {}
        for interest_category in self.get_data().get('interest_categories', []):
            for interest in interest_category.get("interests", []):
                interest_dict.update({interest.get("id"): interest.get("id")})

        return interest_dict

    def render_dictionary(self, form_submission, user_selected_interests=None):

        interests = self.combine_interest_categories(),

        if user_selected_interests:
            interests = {}
            for interest in user_selected_interests:
                interests[interest] = True

        rendered_dictionary_template = json.dumps({
            'email_address': self.get_email_field_template(),
            'merge_fields': self.get_merge_fields_template(),
            'interests': interests,
            'status': 'subscribed',
        })

        rendered_dictionary = Template(rendered_dictionary_template).render(Context(form_submission))
        return rendered_dictionary
